# coding: utf-8

"""
Servizio CodemaOnline
=====================

Modulo per spedizione e ricezione di fax dal servizio codemaonline che
neccessita:

* una casella pop3 cui invia la notifica di un fax in arrivo
* una spedizione di un allegato pdf (altri formati ammessi) attraverso un
  suo servizio (per ora non abilitato)

.. autoclass:: CodemaMsg
   :members:

.. autofunction:: get_messages

.. autofunction:: get_msg

.. _codemaonline: http://www.codemaonline.com/index.php/fax-virtuale.html

Esempi di messaggi ricevuti sono in tests/data


 il modulo prevede l'opzione la cancellazione dei messaggi dopo averli letti.
 tramite la variabile delete passata al metodo

Nota: la cancellazione sul server avviene una volta esplicitata la direttiva Quit(), secondo la documentazione ufficiale di potrebbe eccezione

Settings Codema
---------------

per funzionare correttamente il modulo ha bisogno che siano presenti nei settings.py o nel modulo fax.settings.py

le seguenti variabili per l'accesso alla casella di posta dove risiedono i fax in formato email :

* FAX_CODEMA_HOST     =  'pop.mailserver.it'

* FAX_CODEMA_PASSWORD =  'password'

* FAX_CODEMA_USER     =  'user'

Retrive Codema Fax Online
-------------------------

Command per la ricezione di fax dal servizio codemaonline_ , utilizzo:

python manage.py retrive_codema_fax.py

opzioni:

opzione -d : variabile Booleana per cancellare il messaggi una volta recuperati

opzione -l : variabile Numero per limitare il numero di messaggi recuperare

opzione -v : variabile verbosity di Django. settare a 2 per print di riepilogo
"""

import re
import email
import poplib
import datetime

def get_messages(host, user, pass_, key_file=None, cert_file=None, limit=1000, delete=0 ):
    """Return all messages from host with user/pass

    :param host: the pop3 host
    :param user: the user 
    :param pass_: the password
    :param key_file: a possible key (extension .pem)
    :param cert_file: a possible certificate (extension .pem)
    :param limit: e max number of messages to retrieve
    :param delete: true/false optional delete of messages on server
    """
    
    if key_file:
        pop_instance = poplib.POP3_SSL(host, 995, key_file, cert_file)
        pop_instance.user(user)
    else:
        pop_instance = poplib.POP3(host)
        pop_instance.user(user)
        
    pop_instance.pass_(pass_)
    msg_count, mailbox_size = pop_instance.stat()
    message_list = [get_msg(pop_instance, i+1 ,delete_msg= delete) for i in range(min(msg_count, limit))]

    return message_list, pop_instance
        
def get_msg(pop3_instance, n , delete_msg = False):
    """Return an ``email.message.pop_instanceessage``

    :param pop3_instance: a poplib.POP3 instance
    :param n: the msg number
    :param delete_msg: true/false optional delete of messages on server
    """

    # FIXME: supposes mailbox doesn not change in between
    # Result is in form (response, ['line', ...], octets).

    ret = pop3_instance.retr(n)
    if delete_msg:
        pop3_instance.dele(n)
    
    return email.message_from_string("\n".join(ret[1]))

class CodemaMsg(object):
    """
    Il messaggio ricevuto ha un body primario ed un allegato
    """
    fax_content = None
    """The fax content"""

    status = None
    """Status del messaggio: RECEIVED, SENT, FAILED_RECEIVED, FAILED_SENT"""
    
    info = None
    """un dict con chiavi tali da poter usare Fax(**info):
    sender_number, sender, date FIXME: date non esiste nel modello"""

    def __init__(self, email_msg):
        """parse the message

        :param email_msg: an ``email.message.Message`` instance retrieved from pop
        """
        
        self.msg = email_msg
        payload_count = len(self.msg.get_payload())
        if payload_count == 2:
            # notification + attached fax
            self.notification_msg, self.attach_msg = self.msg.get_payload()
        elif payload_count == 1:
            # presumbly just notification
            self.notification_msg = self.msg.get_payload()[0]
        else:
            raise NotImplementedError('Too many payload...')

        self.parse_notification()
        
    def parse_notification(self):
        """Parse the notification to extract info
        le notifiche di ricezione corretta recano::

           E' stato ricevuto un nuovo fax.

        le notifiche di NON spedizione::

           ERRORE: FAX NON INVIATO - Nessun circuito/canale disponibile
           
        """
        if re.search("E' stato ricevuto un nuovo fax.", self.notification):
            self.status = 'RECEIVED'
            sub_header = dict(self.msg._headers)['Subject']
            info = {'subject_number': sub_header.split("(",1)[-1].rsplit(")",1)[0] }
            for line in self.notification.split('\n'):
                try:
                    key, value = line.split(': ')
                except:
                    continue
                if key == 'Numero Mittente':
                    info['sender_number'] = value.strip()
                elif key == 'Nome Mittente':
                    info['sender'] = value.strip()
                    if info['sender'] == '':
                        info['sender'] =  info['sender_number']
                elif key == 'Data/ora ricez':
                    info['date_sent'] = datetime.datetime.strptime(value.strip(), '%Y/%m/%d %H:%M:%S')
                elif key == 'Totale pagine':
                    info['totale_pagine'] = value.strip()
            self.info = info
                    
            
        elif re.search("FAX NON INVIATO", self.notification):
            self.status = 'FAILED_SENT'

    @property
    def notification(self):
        """The text of the notification received from the fax gateway"""
        return self.notification_msg.get_payload()

    @property
    def attach(self):
        """The received fax as attached pdf document"""
        return self.attach_msg.get_payload(decode=True)

    def write(self, file_name=None):
        """Write the attachment to file
        :param file_name: a possible filename (default to attach_msg.get_filename())
        :return: file_name
        """
        file_name = file_name or self.attach_msg.get_filename()
        with open(file_name, 'wb') as f:
            f.write(self.attach)
        return file_name


    def __str__(self):
        if self.status == 'RECEIVED':
            return "<Fax %s (%s %s)>" % (self.status, self.info['sender_number'],
                                         self.info.get('date', ''))
        else:
            return "<Fax %s>" % self.status
        
